﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Threading;
using System.Threading.Tasks;

namespace Cli
{
	class Program
	{
		static void Main(string[] args)
		{
			var methods = new List<IMethod>
				{
					//new MethodA(),
					new MethodB(),
					new MethodC(),
					//new MethodD(),
					new MethodE(),
					new MethodF(),
				};

			foreach (var method in methods)
			{
				var sw = Stopwatch.StartNew();
				method.Do(100);
				Console.WriteLine("{0} duration : {1}", method.GetType().Name, sw.Elapsed);
			}

			Console.ReadLine();
		}
	}

	public interface IMethod
	{
		void Do(int n);
	}

	public class MethodA : IMethod
	{
		public void Do(int n)
		{
			for (int i = 0; i < n; i++)
			{
				var asynchronousClient = new AsynchronousServiceReference.HelloWorldServiceClient();
				asynchronousClient.HelloWorld("Toto");
			}
		}
	}

	public class MethodB : IMethod
	{
		public void Do(int n)
		{
			var c = n;
			var wh = new ManualResetEvent(false);

			for (int i = 0; i < n; i++)
			{
				var thread = new Thread(() =>
					{
						var asynchronousClient = new AsynchronousServiceReference.HelloWorldServiceClient();
						asynchronousClient.HelloWorld("Toto");
						
						if (Interlocked.Decrement(ref c) == 0)
						{
							wh.Set();
						}
					});

				thread.Start();
			}

			wh.WaitOne();
		}
	}

	public class MethodC : IMethod
	{
		public void Do(int n)
		{
			var c = n;
			var wh = new ManualResetEvent(false);

			for (int i = 0; i < n; i++)
			{
				ThreadPool.QueueUserWorkItem(o =>
					{
						var asynchronousClient = new AsynchronousServiceReference.HelloWorldServiceClient();
						asynchronousClient.HelloWorld("Toto");
						
						if (Interlocked.Decrement(ref c) == 0)
						{
							wh.Set();
						}
					});
			}

			wh.WaitOne();
		}
	}

	public class MethodD : IMethod
	{
		public void Do(int n)
		{

		}
	}

	public class MethodE : IMethod
	{
		public void Do(int n)
		{
			var tasks = new List<Task>();

			for (int i = 0; i < n; i++)
			{
				tasks.Add(Task.Run(() =>
				{
					var asynchronousClient = new AsynchronousServiceReference.HelloWorldServiceClient();
					asynchronousClient.HelloWorld("Toto");
				}));
			}

			Task.WaitAll(tasks.ToArray());
		}
	}

	public class MethodF : IMethod
	{
		public void Do(int n)
		{
			var tasks = new List<Task>();

			var asynchronousClient = new AsynchronousServiceReference.HelloWorldServiceClient();

			for (int i = 0; i < n; i++)
			{
				tasks.Add(asynchronousClient.HelloWorldAsync("Toto"));
			}

			Task.WaitAll(tasks.ToArray());
		}
	}
}
